import 'package:cramr_ui_lib/ads/preloaded_banner_ad.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';
import 'package:provider/provider.dart';

import 'ads_controller.dart';
import 'loading_state.dart';

/// Displays a banner ad that conforms to the widget's size in the layout,
/// and reloads the ad when the user changes orientation.
///
/// Do not use this widget on platforms that AdMob currently doesn't support.
/// For example:
///
/// ```dart
/// if (kIsWeb) {
///   return Text('No ads here! (Yet.)');
/// } else {
///   return MyBannerAd();
/// }
/// ```
///
/// This widget is adapted from pkg:google_mobile_ads's example code,
/// namely the `anchored_adaptive_example.dart` file:
/// https://github.com/googleads/googleads-mobile-flutter/blob/main/packages/google_mobile_ads/example/lib/anchored_adaptive_example.dart
class StreakCongratularionBannerAdWidget extends StatefulWidget {
  const StreakCongratularionBannerAdWidget({super.key});

  @override
  State<StreakCongratularionBannerAdWidget> createState() =>
      _StreakCongratularionBannerAdWidgetState();

  static final adUnitId = defaultTargetPlatform == TargetPlatform.android
      ? 'ca-app-pub-9014261072038331/9426023447'
      : 'ca-app-pub-9014261072038331/9338146111';
}

class _StreakCongratularionBannerAdWidgetState
    extends State<StreakCongratularionBannerAdWidget> {

  BannerAd? _bannerAd;
  LoadingState _adLoadingState = LoadingState.initial;
  late Orientation _currentOrientation;

  @override
  Widget build(BuildContext context) {
    return OrientationBuilder(
      builder: (context, orientation) {
        if (_currentOrientation == orientation &&
            _bannerAd != null &&
            _adLoadingState == LoadingState.loaded) {
          return SizedBox(
            width: _bannerAd!.size.width.toDouble(),
            height: _bannerAd!.size.height.toDouble(),
            child: AdWidget(ad: _bannerAd!),
          );
        }
        // Reload the ad if the orientation changes.
        if (_currentOrientation != orientation) {
          _currentOrientation = orientation;
          _loadAd();
        }
        return const SizedBox();
      },
    );
  }

  @override
  void initState() {
    super.initState();

    final adsController = context.read<AdsController>();
    final ad = adsController
        .takePreloadedAd(StreakCongratularionBannerAdWidget.adUnitId);
    if (ad != null) {
      if (kDebugMode) {
        print("A preloaded banner was supplied. Using it.");
      }
      _showPreloadedAd(ad);
    } else {
      if (kDebugMode) {
        print("A preloaded banner was not supplied. Using it.");
      }
      _loadAd();
    }
  }

  void _showPreloadedAd(PreloadedBannerAd ad) async {
    // It's possible that the banner is still loading (even though it started
    // preloading at the start of the previous screen).
    _adLoadingState = LoadingState.loading;
    try {
      _bannerAd = await ad.ready;
    } on LoadAdError catch (error) {
      _loadAd();
      return;
    }
    if (!mounted) return;

    setState(() {
      _adLoadingState = LoadingState.loaded;
    });
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    _currentOrientation = MediaQuery.of(context).orientation;
  }

  @override
  void dispose() {
    _bannerAd?.dispose();
    super.dispose();
  }

  /// Load (another) ad, disposing of the current ad if there is one.
  Future<void> _loadAd() async {
    if (_adLoadingState == LoadingState.loading ||
        _adLoadingState == LoadingState.disposing) {
      if (kDebugMode) {
        print('MAINPAGE An ad is already being loaded or disposed. Aborting.');
      }
      return;
    }
    _adLoadingState = LoadingState.disposing;
    if (kDebugMode) {
      print('disposing');
    }
    await _bannerAd?.dispose();
    if (kDebugMode) {
      print('_bannerAd disposed');
    }
    if (!mounted) return;
    setState(() {
      _bannerAd = null;
      _adLoadingState = LoadingState.loading;
      if (kDebugMode) {
        print('MainPageBannerAdWidget setState');
      }
    });

    AdSize size;

    size = AdSize.banner;

    if (!mounted) return;

    // assert(Platform.isAndroid || Platform.isIOS,
    //     'AdMob currently does not support ${Platform.operatingSystem}');
    if (kDebugMode) {
      print('About to instantiate ad in _loadAd');
    }
    final AdsController? adsController = context.read<AdsController?>();
    _bannerAd = BannerAd(
      // This is a test ad unit ID from
      // https://developers.google.com/admob/android/test-ads. When ready,
      // you replace this with your own, production ad unit ID,
      // created in https://apps.admob.com/.
      adUnitId: StreakCongratularionBannerAdWidget.adUnitId,
      size: size,
      request: adsController != null
          ? adsController.createAdRequest()
          : const AdRequest(),
      listener: BannerAdListener(
        onAdLoaded: (Ad ad) {
          setState(() {
            // When the ad is loaded, get the ad size and use it to set
            // the height of the ad container.
            _bannerAd = ad as BannerAd;
            _adLoadingState = LoadingState.loaded;
          });
        },
        onAdFailedToLoad: (Ad ad, LoadAdError error) {
          ad.dispose();
        },
        onAdImpression: (Ad ad) {
        },
        onAdClicked: (Ad ad) {
        },
      ),
    );
    if (kDebugMode) {
      print('Bound to load ad in _loadAd');
    }
    return _bannerAd!.load();
  }
}
